-- Pylon: a foundation library; version 0.82(beta); 1998-01-04

-- 
-- Copyright (c) 1997 Franck Arnaud, England, E.U.
-- 
-- Permission is hereby granted, without written agreement and without
-- licence or royalty fees, to use, copy, modify, or distribute this source
-- code or binary programs using it or part of it, subject to the following
-- condition:
-- 
-- The above copyright notice and the following disclaimer must appear in
-- all copies or substantial portions of this source code. (Binary programs
-- derived from the source code can be distributed without restriction or
-- mention of the origin of the source.)
-- 
-- THIS SOFTWARE IS PROVIDED BY FRANCK ARNAUD AND THE AUTHORS ``AS IS'' AND
-- ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-- ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS BE LIABLE TO ANY PARTY FOR 
-- ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
-- DAMAGES ARISING IN ANY WAY OUT OF THE USE OF THIS SOURCE CODE.
-- 

indexing
	cluster: Pylon, Container
	description: "History list -- maximum number of items; first item deleted"
	interface: client

class P_HISTORY_LIST[G]

inherit
	P_DISPENSER[G]
		end
	P_SEQUENCE[G]
		end

creation
	make

feature {P_HISTORY_LIST}
	
	container : P_SEQUENCE_LIST[G]

feature -- from ANY
	
	copy(other : like Current) is
		do
			container.copy(other.container)
		end

	is_equal(other : like Current) : BOOLEAN is
		do
			Result := container.is_equal(other.container)
		end

feature -- from P_CONTAINER

	count : INTEGER is
		do
			Result := container.count
		end

feature {NONE}

	Default_maximum : INTEGER is 100

feature -- Creation
	
	make is
		do
			!!container.make
			set_maximum(Default_maximum)
		end


feature -- Properties
	
	maximum : INTEGER
			-- Max number of item in the container.

	set_maximum(max:INTEGER) is
			-- Set the maximum number of item in the container.
		require
			positive: max > 0
		do
			maximum := max
		end

feature -- Access

	item : G is
		do
			Result := container.item
		end

	set_index(i : INTEGER) is
		do
			container.set_index(i)
		end

feature {P_ITERATOR, P_CONTAINER}
	
	index : INTEGER is
		do
			Result := container.index
		end

feature -- Update
	
	is_writable : BOOLEAN is
			-- Is this container writable?
		do
			Result := container.is_writable
		end

	add(it : G) is
			-- Add at the end of the list, remove first if too big.
		do
			container.add(it)
			if count > maximum then
				container.set_index(1)
				container.remove
			end
		ensure then
			done: (count < maximum) implies (count = old count + 1)
		end

	remove is
			-- This feature need not be used normally.
		do
			container.remove
		end

	reset is
			-- Reset container.
		do
			container.reset
		end

	to_list : P_LIST[G] is
			-- List of elements in container.
		local
			list : P_LINKED_LIST[G]
		do
			!!list.make
			Result := list
			Result.from_list(container)
		end

invariant
	max: maximum > 0
	coherent_count: count = container.count
	count: count <= maximum
	 
end
