-- Pylon: a foundation library; version 0.82(beta); 1998-01-04

-- 
-- Copyright (c) 1997 Franck Arnaud, England, E.U.
-- 
-- Permission is hereby granted, without written agreement and without
-- licence or royalty fees, to use, copy, modify, or distribute this source
-- code or binary programs using it or part of it, subject to the following
-- condition:
-- 
-- The above copyright notice and the following disclaimer must appear in
-- all copies or substantial portions of this source code. (Binary programs
-- derived from the source code can be distributed without restriction or
-- mention of the origin of the source.)
-- 
-- THIS SOFTWARE IS PROVIDED BY FRANCK ARNAUD AND THE AUTHORS ``AS IS'' AND
-- ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-- ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS BE LIABLE TO ANY PARTY FOR 
-- ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
-- DAMAGES ARISING IN ANY WAY OUT OF THE USE OF THIS SOURCE CODE.
-- 

indexing
	cluster: Pylon, Container
	description: "Container which can be sorted"
	interface: abstract

deferred class P_SORTABLE[G]

inherit
	P_CONTAINER[G]
		end

feature {NONE}
	
	order_relation : P_ORDER_RELATION[G]

feature -- Order relation

	has_order : BOOLEAN is
			-- Does this container has an order relation defined?
		do
			Result := order_relation /= Void
		end
	
	set_order(ord_rel : P_ORDER_RELATION[G]) is
			-- Set the order relation which will be used 
			-- when sorting. The relation stays the same when 
			-- the container is reset.
		require
			valid: ord_rel /= Void
		do
			order_relation := ord_rel
		ensure
			keep_reference: has_order
		end

feature -- Sorting

	sort is
			-- Sort the container.
		require
			has_order: has_order
			writable: is_writable
		deferred
		ensure
			sorted: is_sorted
		end

	is_sorted : BOOLEAN is
			-- Is the container sorted?
		require
			has_order: has_order
		deferred
		ensure
			empty_sorted: is_empty implies Result
		end
	
end
