-- Pylon: a foundation library; version 0.82(beta); 1998-01-04

-- 
-- Copyright (c) 1997 Franck Arnaud, England, E.U.
-- 
-- Permission is hereby granted, without written agreement and without
-- licence or royalty fees, to use, copy, modify, or distribute this source
-- code or binary programs using it or part of it, subject to the following
-- condition:
-- 
-- The above copyright notice and the following disclaimer must appear in
-- all copies or substantial portions of this source code. (Binary programs
-- derived from the source code can be distributed without restriction or
-- mention of the origin of the source.)
-- 
-- THIS SOFTWARE IS PROVIDED BY FRANCK ARNAUD AND THE AUTHORS ``AS IS'' AND
-- ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-- ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS BE LIABLE TO ANY PARTY FOR 
-- ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
-- DAMAGES ARISING IN ANY WAY OUT OF THE USE OF THIS SOURCE CODE.
-- 

indexing
	cluster: Pylon
	description: "(internal) general tree node"
	interface: internal, client

class P_TREE_NODE[G]

inherit
	P_NODE[G]
		end

feature {P_TREE_NODE, P_GENERAL_TREE, P_MUTATOR} -- Queries
	
	parent : like Current
			-- Parent node.
	left : like Current
			-- Left sibling.
	right : like Current
			-- Right sibling.
	first_child : like Current
			-- First child.

feature {P_GENERAL_TREE, P_MUTATOR} -- Mutators
	
	set_parent(p : like Current) is
		do
			parent := p
		ensure
			done: parent = p
		end

	set_left(x : like Current) is
		do
			left := x
		ensure
			done: left = x
		end

	set_right(x : like Current) is
		do
			right := x
		ensure
			done: right = x
		end

	set_first_child(x : like Current) is
		do
			first_child := x
		ensure
			done: first_child = x
		end

feature {P_GENERAL_TREE}
	
	is_valid : BOOLEAN is
		do
			Result := (right /= Void implies (right.left = Current and right.parent = parent)) -- right_sibling
					and then (left /= Void implies (left.right = Current and left.parent = parent)) -- left_sibling
					and then ((left = Void and parent /= Void) implies parent.first_child = Current) -- first child
		end

invariant
	child: first_child /= Void implies first_child.parent = Current
	sole_root: parent = Void implies (right = Void and left = Void)
	good_family: -- no node twice in the same tree

end
